import IScrollView from "../Component/Script/IScrollView";
import { getDivision, setGradeIcon, setRank, setText, createImage } from "./Util/Tool";

const { ccclass, property } = cc._decorator;

@ccclass
export class Rank extends cc.Component {
    @property(IScrollView)
    scrollView: IScrollView = null;

    @property(cc.Node)
    myNode: cc.Node = null;

    @property(cc.Node)
    noList: cc.Node = null;

    @property(cc.Node)
    disable: cc.Node = null;

    friendDatas: any[];

    keys: string[];

    selfOpenid: string;

    userData: any;

    onLoad() {
        this.scrollView.isFetch = false;
        this.scrollView.stopAutoScroll()
    }

    start() {
        wx.onMessage(data => {
            console.log("接收主域发来消息：", data)
            switch (data.messageType) {
                case 1:
                    this.keys = data.keys;
                    this.selfOpenid = data.openid;
                    this.getFriend(data.keys);
                    this.userData = data.user;
                    break;
                case 2:
                    this.showRankByRankType(data.type);
                    break;
                case 3:
                    this.disable.active = !data.canScroll;
                    break;
                case 4:
                    console.log(data);
                    this.scrollView.node.emit(data.touchType, data.e);
                    break;
            }
        });
    }

    setFriend(type, score) {
        wx.getUserCloudStorage({
            keyList: [type],
            success: (getres) => {
                console.log('getres', getres)
                //取出所有好友数据
                wx.setUserCloudStorage({
                    KVDataList: [{ key: type, value: score }],
                    success: res => {
                        console.log("wx.setUserCloudStorage success", res);

                    },
                    fail: res => {
                        console.log("wx.setUserCloudStorage fail", res);
                    },
                });
            },
            fail: (res) => {
            }
        });
    }
    getFriend(keys: any[]) {
        wx.getUserInfo({
            openIdList: ['selfOpenId'],
            success: (userRes) => {
                console.log('success', userRes.data)
                //取出所有好友数据
                wx.getFriendCloudStorage({
                    keyList: keys,
                    success: res => {
                        console.log("wx.getFriendCloudStorage success", res);
                        this.friendDatas = res.data;
                        this.showRankByRankType(RankType.DIVISION);
                        this.noList.active = !this.scrollView.dataProvider || !this.scrollView.dataProvider.length;
                        // const temp = [{ type: RankType.ROOM_COUNT, nickName: '蒋介石', score: 100 }]
                        // this.scrollView.dataProvider = temp;
                    },
                    fail: res => {
                        console.log("wx.getFriendCloudStorage fail", res);
                    },
                });
            },
            fail: (res) => {
            }
        });
    }

    showRankByRankType(type: number) {
        if (!this.friendDatas) return;
        switch (type) {
            case RankType.DIVISION:
                this.filterByDivision();
                break;
            case RankType.ROOM_COUNT:
                this.filterByRoomCount();
                break;
            case RankType.WEALTH_COUPON:
                this.filterByCoupon();
                break;
        }
        let selfRank = -1;
        const selfData = this.friendDatas.filter((v, i) => {
            if (v.openid === this.selfOpenid) selfRank = i;
            return v.openid === this.selfOpenid;
        })[0];
        console.log('selfData', selfData);
        if (selfData) {
            const kvData = selfData.KVDataList.filter(v => v.key === selfData.dataKey)[0].value;
            if (kvData) {
                console.log('kvdata', kvData);
                this.setCurRank(selfData, selfRank);
            } else {
                this.seSelf(type, this.userData, -1);
            }
        } else {
            this.seSelf(type, this.userData, -1);
        }
    }

    async seSelf(type, data: any, rank: number) {
        const str = rank < 0 ? '未上榜' : `${rank + 1}`;
        const ranklabel = this.myNode.getChildByName('rank').getComponent(cc.Label);
        setRank(rank + 1, ranklabel);
        setText(this.myNode, 'curRank', `当前排名：${str}`);
        createImage(data.headimgUrl, this.myNode.getChildByName('head').getComponent(cc.Sprite));
        const info = this.myNode.getChildByName('info');
        const iconNode = info.getChildByName('gradeIcon');
        switch (type) {
            case RankType.DIVISION:
                iconNode.active = true;
                const json = await getDivision(this.userData.division, this.userData.divisionLevel);
                const sprite = await setGradeIcon(json.division, json.level, 's');
                iconNode.getComponent(cc.Sprite).spriteFrame = sprite;
                setText(info, 'gradeName', `${json.name}·${this.userData.divisionStar}星`);
                break;
            case RankType.ROOM_COUNT:
                setText(info, 'gradeName', `对局数量：${0}`);
                iconNode.active = false;
                break;
            case RankType.WEALTH_COUPON:
                setText(info, 'gradeName', `红包券数量：${0}`);
                iconNode.active = false;
                break;
        }
    }
    async setCurRank(data: any, rank: number) {
        const str = rank < 0 ? '未上榜' : `${rank + 1}`;
        const ranklabel = this.myNode.getChildByName('rank').getComponent(cc.Label);
        setRank(rank + 1, ranklabel);
        setText(this.myNode, 'curRank', `当前排名：${str}`);
        createImage(data.avatarUrl, this.myNode.getChildByName('head').getComponent(cc.Sprite));
        const info = this.myNode.getChildByName('info');
        const iconNode = info.getChildByName('gradeIcon');
        const kvData = data.KVDataList.filter(v => v.key === data.dataKey)[0].value;
        switch (data.type) {
            case RankType.DIVISION:
                iconNode.active = true;
                const json = await getDivision(JSON.parse(kvData).division, JSON.parse(kvData).divisionLevel);
                const sprite = await setGradeIcon(json.division, json.level, 's');
                iconNode.getComponent(cc.Sprite).spriteFrame = sprite;
                setText(info, 'gradeName', `${json.name}·${JSON.parse(kvData).divisionStar}星`);
                break;
            case RankType.ROOM_COUNT:
                setText(info, 'gradeName', `对局数量：${kvData}`);
                iconNode.active = false;
                break;
            case RankType.WEALTH_COUPON:
                setText(info, 'gradeName', `红包券数量：${kvData}`);
                iconNode.active = false;
                break;
        }
    }

    filterByCoupon() {
        const coupons = this.friendDatas.sort((a, b) => {
            if (!b.KVDataList || !b.KVDataList.length) return -1;
            if (!a.KVDataList || !a.KVDataList.length) return 1;
            const acoupon = parseInt(a.KVDataList.filter(v => v.key === this.keys[1])[0].value);
            const bcoupon = parseInt(b.KVDataList.filter(v => v.key === this.keys[1])[0].value);
            return bcoupon - acoupon;
        });
        for (const c of coupons) {
            c.type = RankType.WEALTH_COUPON;
            c.dataKey = this.keys[1];
        }
        this.scrollView.scrollToTop();
        this.scrollView.dataProvider = coupons.filter(c => c.KVDataList && c.KVDataList.length && c.KVDataList.filter(v => v.key === this.keys[1])[0]);
    }
    filterByRoomCount() {
        const counts = this.friendDatas.sort((a, b) => {
            if (!b.KVDataList || !b.KVDataList.length) return -1;
            if (!a.KVDataList || !a.KVDataList.length) return 1;
            const acount = parseInt(a.KVDataList.filter(v => v.key === this.keys[2])[0].value);
            const bcount = parseInt(b.KVDataList.filter(v => v.key === this.keys[2])[0].value);
            return bcount - acount;
        });
        for (const c of counts) {
            c.type = RankType.ROOM_COUNT;
            c.dataKey = this.keys[2];
        }
        this.scrollView.scrollToTop();
        this.scrollView.dataProvider = counts.filter(c => c.KVDataList && c.KVDataList.length && c.KVDataList.filter(v => v.key === this.keys[2])[0]);;
    }

    filterByDivision() {
        const divisions = this.friendDatas.sort((a: any, b: any) => {
            if (!b.KVDataList || !b.KVDataList.length) return -1;
            if (!a.KVDataList || !a.KVDataList.length) return 1;
            const aDivision = JSON.parse(a.KVDataList.filter(v => v.key === this.keys[0])[0].value);
            const bDivision = JSON.parse(b.KVDataList.filter(v => v.key === this.keys[0])[0].value);
            if (aDivision.division !== bDivision.division) return bDivision.division - aDivision.division;
            if (aDivision.divisionLevel !== bDivision.divisionLevel) return bDivision.divisionLevel - aDivision.divisionLevel;
            return bDivision.divisionStar - aDivision.divisionStar;
        });
        for (const d of divisions) {
            d.type = RankType.DIVISION;
            d.dataKey = this.keys[0];
        }
        this.scrollView.scrollToTop();
        this.scrollView.dataProvider = divisions.filter(c => c.KVDataList && c.KVDataList.length && c.KVDataList.filter(v => v.key === this.keys[0])[0]);;
    }
}

export enum RankType {
    DIVISION = 1, // 段位排行
    WEALTH_COUPON, // 财富之礼券
    ROOM_COUNT, // 局数榜单
}